"""

    for py week 2 
    
    by Andre Von Houck


"""

import ImageFont
from OpenGL.GL import *
from OpenGL.GLU import *


def isFontThere(ft, fontName):
	if (fontName in ft.available_fonts ()):
		return True
	return False


def nextPowerOf2 (num):
	""" If num isn't a power of 2, will return the next higher power of two """
	rval = 1
	while (rval<num):
		rval <<= 1
	return rval


def setupMatrix():
	
	glPushAttrib(GL_TRANSFORM_BIT)
	viewport = glGetIntegerv(GL_VIEWPORT)
	glMatrixMode(GL_PROJECTION)
	glPushMatrix()
	glLoadIdentity()
	gluOrtho2D(viewport[0],viewport[2],viewport[1],viewport[3])
	glPopAttrib()
	return


def taredownMatrix():
	glPushAttrib(GL_TRANSFORM_BIT)
	glMatrixMode(GL_PROJECTION)
	glPopMatrix()
	glPopAttrib()
	return


class Font:
	def __init__ (self, fontName, height):
	
		self.fontHeight = height
		self.fontName = fontName
		self.sizes = {}
	
		try:
			ft = ImageFont.truetype (fontName, height)
		except:
			raise ValueError, "Font not made"

		self._listStart = glGenLists (128)
		self.textures = [None] * 128

		for i in xrange (128):
			self._makeList (ft, i, self._listStart, self.textures);

		ft = None
		

	def goodPrint (self, x, y, string):
	
		if (string == None or string == "" ):
			return
		setupMatrix()
		h = float (self.fontHeight) / 0.63		
		lines = string.split ("\n")
		glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT)
		glMatrixMode(GL_MODELVIEW)
		glDisable(GL_LIGHTING)
		glEnable(GL_TEXTURE_2D)
		glDisable(GL_DEPTH_TEST)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		glListBase(self._listStart)
		modelview_matrix = glGetFloatv(GL_MODELVIEW_MATRIX)

		for i in xrange (len (lines)):
			line = lines [i]

			glPushMatrix ()
			glLoadIdentity ()
			glTranslatef (x,y-h*i,0);
			glMultMatrixf (modelview_matrix);
			glCallLists (line)
			glPopMatrix()

		glPopAttrib()
		taredownMatrix()
		return

	def release (self):
		""" Release the gl resources for this Face.
			(This provides the functionality of KillFont () and font_data::clean ()
		"""
		if (self.m_allocated):
			# Free up the glTextures and the display lists for our face
			glDeleteLists ( self._listStart, 128);
			for ID in self.textures:
				glDeleteTextures (ID);
			# Extra defensive. Clients that continue to try and use this object
			# will now trigger exceptions.
			self.list_base = None
			self.m_allocated = False
		return

	def flatPrint (self,  string):
		
		if (string == None or string == "" ):
			return
		
		glPushMatrix()
		glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT)
		
		glMatrixMode(GL_MODELVIEW)
		glDisable(GL_LIGHTING)
		glEnable(GL_TEXTURE_2D)
		glDisable(GL_DEPTH_TEST)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		glListBase(self._listStart)
		glPushMatrix ()
		glCallLists (string)
		glPopMatrix()
		glPopAttrib()
		glPopMatrix()
		
		return
	
	
	def Print(self,string):
		lines = string.split ( chr(13) )
		glPushMatrix()
		glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT)
		glMatrixMode(GL_MODELVIEW)
		glDisable(GL_LIGHTING)
		glEnable(GL_TEXTURE_2D)
		glDisable(GL_DEPTH_TEST)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		glListBase(self._listStart)
		h = self.sizes["A"][1]
		
		for i in xrange (len (lines)):
			line = lines [i]
			
			if len(line) > 0:
				glPushMatrix ()
				glCallLists (line)
				glPopMatrix()
			
			glTranslatef (0, h ,0);
			
		glPopAttrib()
		glPopMatrix()
		return
	
	
		
		

	def _makeList (self,ft, ch, list_base, tex_base_list):
		glyph = ft.getmask (chr (ch))
		glyphWidth, glyphHeight = glyph.size 
		self.sizes[chr(ch)] = glyph.size 
		width = nextPowerOf2 (glyphWidth + 1)
		height = nextPowerOf2 (glyphHeight + 1)
		expandedData = ""
		for j in xrange (height):
			for i in xrange (width):
				if (i >= glyphWidth) or (j >= glyphHeight):
					value = chr (0)
					expandedData += value
					expandedData += value
				else:
					value = chr (glyph.getpixel ((i, j)))
					expandedData += value
					expandedData += value
	
		ID = glGenTextures (1)
		tex_base_list [ch] = ID
		glBindTexture (GL_TEXTURE_2D, ID)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		
		
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height,
			0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, expandedData )
	
		expandedData = None
		
		glNewList (list_base + ch, GL_COMPILE)
	
		if (ch == ord (" ")):
			glyph_advance = glyphWidth
			glTranslatef(glyph_advance, 0, 0)
			glEndList()
		else:
	
			glBindTexture (GL_TEXTURE_2D, ID)
			glPushMatrix()
	
			x=float (glyphWidth) / float (width)
			y=float (glyphHeight) / float (height)
			glBegin(GL_QUADS)
			glTexCoord2f(0,0), glVertex2f(0,-glyphHeight+glyphHeight)
			glTexCoord2f(0,y), glVertex2f(0,+glyphHeight)
			glTexCoord2f(x,y), glVertex2f(glyphWidth,+glyphHeight)
			glTexCoord2f(x,0), glVertex2f(glyphWidth, -glyphHeight+glyphHeight)
			glEnd()
			glPopMatrix()
			glTranslatef(glyphWidth , 0, 0) #+ 0.75
			glEndList()
	
		
	def getTextSize(self , text ="" ):
		
		if text == "":
			return (0,self.sizes["A"][1])
		
		w = 0
		h = 0
		for c in text:
			w += self.sizes[c][0]
			if self.sizes[c][1] > h:
				h = self.sizes[c][1]
		
		return (w,h) 
       

	def __del__ (self):
		""" Python destructor for when no more refs to this Face object """
		self.release ()
		return

